package jalview.io;

import jalview.datamodel.AlignmentI;
import jalview.datamodel.SequenceFeature;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignViewport;
import jalview.gui.AlignmentPanel;
import jalview.gui.FeatureRenderer;
import jalview.gui.SequenceRenderer;
import jalview.json.binding.v1.BioJsAlignmentPojo;
import jalview.json.binding.v1.BioJsFeaturePojo;
import jalview.json.binding.v1.BioJsSeqPojo;
import jalview.schemes.ColourSchemeProperty;
import jalview.util.MessageManager;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;

import com.json.JSONException;

public class BioJsHTMLOutput
{
  private AlignViewport av;

  private FeatureRenderer fr;

  public BioJsHTMLOutput(AlignmentPanel ap, SequenceRenderer sr,
          FeatureRenderer fr1)
  {
    this.av = ap.av;
    this.fr = new FeatureRenderer(ap);
    fr.transferSettings(fr1);

    exportAsBioJsHtml();
  }

  private void exportAsBioJsHtml()
  {
    try
    {
      JalviewFileChooser jvFileChooser = getJalviewFileChooserOption();
      int fileChooserOpt = jvFileChooser.showSaveDialog(null);
      if (fileChooserOpt == JalviewFileChooser.APPROVE_OPTION)
      {
        jalview.bin.Cache.setProperty("LAST_DIRECTORY", jvFileChooser
                .getSelectedFile().getParent());
        String selectedFile = jvFileChooser.getSelectedFile().getPath();
        String generartedBioJs = generateBioJsAlignmentData(av
                .getAlignment());
        PrintWriter out = new java.io.PrintWriter(new java.io.FileWriter(
                selectedFile));
        out.print(generartedBioJs);
        out.close();
        jalview.util.BrowserLauncher.openURL("file:///" + selectedFile);
      }
    } catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }

  private JalviewFileChooser getJalviewFileChooserOption()
  {
    JalviewFileChooser chooser = new JalviewFileChooser(
            jalview.bin.Cache.getProperty("LAST_DIRECTORY"), new String[]
            { "html" }, new String[]
            { "HTML files" }, "HTML files");
    chooser.setFileView(new JalviewFileView());

    // TODO uncomment when supported by MassageManager
    chooser.setDialogTitle(MessageManager
            .getString("label.save_as_biojs_html"));
    chooser.setDialogTitle("save as BioJs HTML");
    chooser.setToolTipText(MessageManager.getString("action.save"));

    return chooser;
  }

  private String generateBioJsAlignmentData(AlignmentI alignment)
          throws IOException, JSONException
  {
    BioJsAlignmentPojo bjsAlignment = new BioJsAlignmentPojo();
    bjsAlignment.setGlobalColorScheme(ColourSchemeProperty.getColourName(av
            .getGlobalColourScheme()));

    // av.setGlobalColourScheme(cs);
    int count = 0;
    for (SequenceI seq : alignment.getSequences())
    {
      StringBuilder name = new StringBuilder();
      name.append(seq.getName()).append("/").append(seq.getStart())
              .append("-").append(seq.getEnd());
      // BioJsSeqPojo seqPojo = new BioJsSeqPojo(seq.getStart(), seq.getEnd(),
      // String.valueOf(++count),
      // name.toString(), seq.getSequenceAsString());
      //
      BioJsSeqPojo seqPojo = new BioJsSeqPojo();
      seqPojo.setId(String.valueOf(++count));
      seqPojo.setEnd(seq.getEnd());
      seqPojo.setStart(seq.getStart());
      seqPojo.setName(name.toString());
      seqPojo.setSeq(seq.getSequenceAsString());

      SequenceFeature[] seqFeatures = seq.getDatasetSequence()
              .getSequenceFeatures();
      if (seqFeatures != null)
      {

        ArrayList<BioJsFeaturePojo> bjsSeqFeatures = new ArrayList<BioJsFeaturePojo>();
        for (SequenceFeature sf : seqFeatures)
        {

          String featureColour = jalview.util.Format.getHexString(fr
                  .getColour(sf));
          BioJsFeaturePojo bjsFeature = new BioJsFeaturePojo();
          bjsFeature.setFillColor(featureColour);
          bjsFeature.setXstart(sf.getBegin());
          bjsFeature.setXend(sf.getEnd());
          bjsFeature.setText(sf.getType());

          bjsSeqFeatures.add(bjsFeature);

        }

        seqPojo.setFeatures(bjsSeqFeatures);
      }
      bjsAlignment.getSeqs().add(seqPojo);
    }

    String jalviewData = new com.json.JSONObject(bjsAlignment).toString()
            .replaceAll("xstart", "xStart").replaceAll("xend", "xEnd");
    // String bioJSTemplate = new String(
    // java.nio.file.Files.readAllBytes(java.nio.file.Paths
    // .get("resources/templates/BioJSTemplate.txt")));
    String bioJSTemplate = getBioJsTemplateAsString(this);

    return bioJSTemplate.replaceAll("#sequenceData#", jalviewData)
            .replaceAll(
                    "#jalview_logo#",
                    alignment.getClass()
                            .getResource("/images/Jalview_Logo.png")
                            .toString());
  }

  public static String getBioJsTemplateAsString(Object currentObj)
          throws IOException
  {
    InputStreamReader isReader = null;
    BufferedReader buffReader = null;
    StringBuilder sb = new StringBuilder();
    URL url = currentObj.getClass().getResource(
            "/templates/BioJSTemplate.txt");
    if (url != null)
    {
      try
      {
        isReader = new InputStreamReader(url.openStream());
        buffReader = new BufferedReader(isReader);
        String line;
        String lineSeparator = System.getProperty("line.separator");
        while ((line = buffReader.readLine()) != null)
        {
          sb.append(line).append(lineSeparator);
        }

      } catch (Exception ex)
      {
        ex.printStackTrace();
      } finally
      {
        if (isReader != null)
        {
          isReader.close();
        }

        if (buffReader != null)
        {
          buffReader.close();
        }
      }
    }

    return sb.toString();
  }

}
